
public class Personne {
	
	private static final boolean homme = true;   //pour utiliser directement homme ou femme au lieu de true and false, ce qui diminuerait la clart
	private static final boolean femme = false;
	
	private String nom;
	private String prenom;
	private int anneeNaissance;
	private static int anneeCourante;      //mot clef static car l'annee courante ne depend pas individuellement de chaque personne.
	private boolean sexe;
	private boolean estMarie;	
	private Personne conjoint;
	private Compte compte;
	
	
	Personne(String nom, String prenom, int anneeN, boolean s){
		this.nom = nom;
		this.prenom = prenom;
		this.anneeNaissance = anneeN;
		this.sexe = s;
		this.estMarie = false;
		this.conjoint = null;
	}
	
	Personne(String nom, String prenom, int anneeN, boolean s, Personne p){
		this(nom, prenom, anneeN, s);
		this.estMarie = true;
		this.conjoint = p;
	}
	
	Personne(String nom, String prenom, int anneeN, boolean s, int somme){
		this(nom, prenom, anneeN, s);
		this.estMarie = false;
		this.compte.setSolde(somme);
	}
	
	public String infos(){		
		String statutH = this.estMarie? "mari" : "clibataire";    //syntaxe astucieuse
		String statutF = this.estMarie? "marie" : "clibataire";
		String conj = this.estMarie? " avec " + this.conjoint.nom + " " + this.conjoint.prenom + "." : "." ; 
		
		if(this.sexe == homme)
			return "Bonjour, je m'appelle " + prenom + " " + nom + ", je suis n en " + this.anneeNaissance + " et je suis " + statutH + conj;
		else
			return "Bonjour, je m'appelle " + prenom + " " + nom + ", je suis ne en " + this.anneeNaissance + " et je suis " + statutF + conj;
	}
	
	public static void setAnneeCourange(int n){
		Personne.anneeCourante = n;
	}
	
	public void incrementeAnneeCourante(){
		Personne.anneeCourante ++;
	}
	
	public int age(){
		return Personne.anneeCourante - this.anneeNaissance;
	}
	
	public boolean peutSeMarier(){
		return (this.age() >= 18 && !this.estMarie);
	}
	
	public boolean marier(Personne p){
		if (this.peutSeMarier() && p.peutSeMarier()){
			this.estMarie = true;
			p.estMarie = true;			
			this.conjoint = p;
			p.conjoint = this;
			System.out.println("Flicitations aux maris: " + this.nom + " " + this.prenom + " et " + p.nom + " " + p.prenom +".");
			return true;
		}
		else return false;
	}
	
	public boolean marieAvec (Personne p){
		return (this.conjoint == p);
	}
	
	public static boolean marieAvec(Personne p1, Personne p2){
			return (p1.conjoint == p2);
	}
	
	public boolean divorcer(){
		if(this.estMarie){
			this.estMarie = false;
			this.conjoint.estMarie = false;
			this.conjoint.conjoint = null;
			this.conjoint = null;
			System.out.println("Et un divorce de plus.");
			return true;
		}
		else return false;
	}
	
	public Personne enfanter(String prenom, char sex) throws ErrorSexeFalse{
			if (this.sexe == homme)
				throw new ErrorSexeFalse();
		
		switch(sex){
			case 'M': return new Personne(this.nom, prenom, Personne.anneeCourante, homme);
			case 'F': return new Personne(this.nom, prenom, Personne.anneeCourante, femme);
			default: return null;
		}	 
	}
	
	public static void main(String[] args) throws ErrorSexeFalse {
		
		try{
		Personne.setAnneeCourange(2012);
		
		Personne p1 = new Personne("Bertrand", "Jean", 1978, Personne.homme);
		Personne p2 = new Personne("Dutivier", "Flora", 1980, Personne.femme);
		System.out.println(p1.age());
		System.out.println(p2.age());
		System.out.println(p1.infos());
		System.out.println(p2.infos());
		
		p1.marier(p2);
		System.out.println(p1.infos());
		System.out.println(p2.infos());
		
		p1.divorcer();
		System.out.println(p1.infos());
		Personne p3 = p1.enfanter("Russel",'M');
		Personne p4 = p2.enfanter("Robert", 'M');
		System.out.println(p3.infos());
		}
		catch(ErrorSexeFalse e){
			System.out.println("Seule une femme peut enfanter.");
			System.exit(-1);
		}
	}
}
